import os
import pandas as pd

from django.shortcuts import render,get_object_or_404, redirect
from account.models import Account 
from django.core.paginator import Paginator, PageNotAnInteger, EmptyPage
from django.http import HttpResponse 
from django.contrib.auth.models import Group 
from django.contrib.auth.decorators import login_required 
from django.http import HttpResponseForbidden
from django.contrib import messages

from config.EMAILSERVICE import EmailService
from config.security.password import generate_password
from django.core.exceptions import ObjectDoesNotExist
from django.core.files.storage import FileSystemStorage
from django.conf import settings

@login_required
def backoffice(request):
    if request.user.is_admin_user or request.user.is_super_user:
        return render(request, "backoffice/index.html")
    else:
        return HttpResponseForbidden("You are not authorized to view this page.")

@login_required
def add_staff(request):
    if request.user.is_admin_user or request.user.is_super_user:
        if request.method == 'POST':
            first_name = request.POST.get('first_name')
            second_name = request.POST.get('second_name')
            email = request.POST.get('email')
            phone_number = request.POST.get('phone_number')
            gender = request.POST.get('gender')
            dob = request.POST.get('dob')
            age = request.POST.get('age')
            marital_status = request.POST.get('marital_status')
            location = request.POST.get('location')
            country = request.POST.get('country')
            branch = request.POST.get('branch')
            employment_type = request.POST.get('employment_type')
            kra_pin = request.POST.get('kra_pin')
            status = request.POST.get('status')
            education = request.POST.get('education')
            nationality = request.POST.get('nationality')
            next_of_kin_name = request.POST.get('next_of_kin_name')
            next_of_kin_phone = request.POST.get('next_of_kin_phone')
            emergency_contact = request.POST.get('emergency_contact')
            emergency_contact_name = request.POST.get('emergency_contact_name')
            emergency_contact_phone = request.POST.get('emergency_contact_phone')
            emergency_contact_relationship = request.POST.get('emergency_contact_relationship')
            secondary_emergency_contact_name = request.POST.get('secondary_emergency_contact_name')
            secondary_emergency_contact_phone = request.POST.get('secondary_emergency_contact_phone')
            secondary_emergency_contact_relationship = request.POST.get('secondary_emergency_contact_relationship')

            # Retrieve permission values using request.POST.getlist()
            is_admin_user = 'is_admin_user' in request.POST
            is_super_user = 'is_super_user' in request.POST
            is_sales = 'is_sales' in request.POST
            is_purchases = 'is_purchases' in request.POST
            is_profiling = 'is_profiling' in request.POST
            is_production = 'is_production' in request.POST
            is_inventory = 'is_inventory' in request.POST
            is_pos = 'is_pos' in request.POST
            is_accounting = 'is_accounting' in request.POST

            try:
                # Check if email already exists
                existing_email = Account.objects.get(email=email)
                if existing_email:
                    messages.error(request, f'An account with email {email} already exists.')
                    return render(request, "backoffice/staff/add_staff.html")
            except ObjectDoesNotExist:
                pass

            try:
                # Check if phone number already exists
                existing_phone = Account.objects.get(phone_number=phone_number)
                if existing_phone:
                    messages.error(request, f'An account with phone number {phone_number} already exists.')
                    return render(request, "backoffice/staff/add_staff.html")
            except ObjectDoesNotExist:
                pass

            try:
                # Create the Account instance and save it
                account = Account.objects.create(
                    first_name=first_name,
                    second_name=second_name,
                    email=email,
                    phone_number=phone_number,
                    gender=gender,
                    dob=dob,
                    age=age,
                    marital_status=marital_status,
                    location=location,
                    country=country,
                    employment_type=employment_type,
                    kra_pin=kra_pin,
                    status=status,
                    education=education,
                    nationality=nationality,
                    next_of_kin_name=next_of_kin_name,
                    next_of_kin_phone=next_of_kin_phone,
                    emergency_contact=emergency_contact,
                    emergency_contact_name=emergency_contact_name,
                    emergency_contact_phone=emergency_contact_phone,
                    emergency_contact_relationship=emergency_contact_relationship,
                    secondary_emergency_contact_name=secondary_emergency_contact_name,
                    secondary_emergency_contact_phone=secondary_emergency_contact_phone,
                    secondary_emergency_contact_relationship=secondary_emergency_contact_relationship,
                    is_staff=is_admin_user or is_super_user,
                    is_admin_user=is_admin_user,
                    is_super_user=is_super_user,
                    is_sales=is_sales,
                    is_purchases=is_purchases,
                    is_profiling=is_profiling,
                    is_production=is_production,
                    is_inventory=is_inventory,
                    is_pos=is_pos
                )
                # Handle profile image upload
                profile_image = request.FILES.get('profile_image')
                if profile_image:
                    fs = FileSystemStorage(location=os.path.join(settings.MEDIA_ROOT, 'user_profiles/images'))
                    filename = fs.save(profile_image.name, profile_image)
                    account.profile_image = os.path.join('user_profiles/images', filename)

                # Handle document upload
                documents = request.FILES.getlist('documents')
                if documents:
                    fs = FileSystemStorage(location=os.path.join(settings.MEDIA_ROOT, 'user_profiles/documents'))
                    document_paths = []
                    for document in documents:
                        filename = fs.save(document.name, document)
                        document_paths.append(os.path.join('user_profiles/documents', filename))
                    account.documents = ', '.join(document_paths)

                # Assign the user to groups based on permissions
                groups = []
                if is_admin_user or is_super_user:
                    groups = Group.objects.all()
                else:
                    if is_sales:
                        groups.append(Group.objects.get(name='SALES'))
                    if is_profiling:
                        groups.append(Group.objects.get(name='PROFILING'))
                    if is_production:
                        groups.append(Group.objects.get(name='PRODUCTION'))
                    if is_inventory:
                        groups.append(Group.objects.get(name='INVENTORY'))
                    if is_pos:
                        groups.append(Group.objects.get(name='POS'))
                    if is_accounting:
                        groups.append(Group.objects.get(name='ACCOUNTING'))

                account.groups.set(groups)

                # Set password for the user
                password = generate_password(first_name, second_name, gender)
                account.set_password(password)
                account.save()

                # Success message
                messages.success(request, f'Account created for {first_name}!')

                # Send email with login credentials
                email_service = EmailService()
                user_email = email
                subject = "MABATI IQ LOGIN DETAILS"

                message = f"Hello {first_name},\n\nWelcome to MabatiCrm.com ERP!\n\n"
                message += f"Your password is: {password}\n"
                message += f"Your username is: {email}\n\n"

                if groups:
                    # If the user is assigned to groups, list their roles
                    message += "Your role(s) in the system:\n"
                    for role in groups:
                        message += f"- {role}\n"

                message += "\nClick here to visit the website: {email_service.host_name}\n"
                message += "\nThank you for joining EasyCom ERP!"

                email_service.send_mail(user_email, subject, message)
                print("Email sent.")

            except Exception as e:
                messages.error(request, f'An error occurred: {str(e)}')

            return render(request, "backoffice/staff/add_staff.html")
        return render(request, "backoffice/staff/add_staff.html")
    else:
        return render(request, "backoffice/staff/add_staff.html")
     

@login_required
def edit_staff(request, staff_id):
    # Check user permissions
    if request.user.is_admin_user or request.user.is_super_user:
        # Retrieve the staff member object
        staff_member = get_object_or_404(Account, id=staff_id)

        if request.method == 'POST':
            try:
                # Retrieve form data
                first_name = request.POST.get('first_name')
                second_name = request.POST.get('second_name')
                email = request.POST.get('email')
                phone_number = request.POST.get('phone_number')
                gender = request.POST.get('gender')
                dob = request.POST.get('dob')
                age = request.POST.get('age')
                marital_status = request.POST.get('marital_status')
                location = request.POST.get('location')
                country = request.POST.get('country')
                employment_type = request.POST.get('employment_type')
                kra_pin = request.POST.get('kra_pin')
                status = request.POST.get('status')
                education = request.POST.get('education')
                nationality = request.POST.get('nationality')
                next_of_kin_name = request.POST.get('next_of_kin_name')
                next_of_kin_phone = request.POST.get('next_of_kin_phone')
                emergency_contact = request.POST.get('emergency_contact')
                emergency_contact_name = request.POST.get('emergency_contact_name')
                emergency_contact_phone = request.POST.get('emergency_contact_phone')
                emergency_contact_relationship = request.POST.get('emergency_contact_relationship')
                secondary_emergency_contact_name = request.POST.get('secondary_emergency_contact_name')
                secondary_emergency_contact_phone = request.POST.get('secondary_emergency_contact_phone')
                secondary_emergency_contact_relationship = request.POST.get('secondary_emergency_contact_relationship')

                # Retrieve permission values using request.POST.getlist()
                is_admin_user = 'is_admin_user' in request.POST
                is_super_user = 'is_super_user' in request.POST
                is_sales = 'is_sales' in request.POST
                is_purchases = 'is_purchases' in request.POST
                is_profiling = 'is_profiling' in request.POST
                is_production = 'is_production' in request.POST
                is_inventory = 'is_inventory' in request.POST
                is_pos = 'is_pos' in request.POST
                is_accounting = 'is_accounting' in request.POST

                # Update the staff member object with new data
                staff_member.first_name = first_name
                staff_member.second_name = second_name
                staff_member.email = email
                staff_member.phone_number = phone_number
                staff_member.gender = gender
                staff_member.dob = dob
                staff_member.age = age
                staff_member.marital_status = marital_status
                staff_member.location = location
                staff_member.country = country
                staff_member.employment_type = employment_type
                staff_member.kra_pin = kra_pin
                staff_member.status = status
                staff_member.education = education
                staff_member.nationality = nationality
                staff_member.next_of_kin_name = next_of_kin_name
                staff_member.next_of_kin_phone = next_of_kin_phone
                staff_member.emergency_contact = emergency_contact
                staff_member.emergency_contact_name = emergency_contact_name
                staff_member.emergency_contact_phone = emergency_contact_phone
                staff_member.emergency_contact_relationship = emergency_contact_relationship
                staff_member.secondary_emergency_contact_name = secondary_emergency_contact_name
                staff_member.secondary_emergency_contact_phone = secondary_emergency_contact_phone
                staff_member.secondary_emergency_contact_relationship = secondary_emergency_contact_relationship
                staff_member.is_admin_user = is_admin_user
                staff_member.is_super_user = is_super_user
                staff_member.is_sales = is_sales
                staff_member.is_purchases = is_purchases
                staff_member.is_profiling = is_profiling
                staff_member.is_production = is_production
                staff_member.is_inventory = is_inventory
                staff_member.is_pos = is_pos
                staff_member.is_accounting = is_accounting

                staff_member.save()

                # Assign the user to groups based on permissions
                groups = []
                if is_admin_user or is_super_user:
                    groups = Group.objects.all()
                else:
                    if is_sales:
                        groups.append(Group.objects.get(name='SALES'))
                    if is_profiling:
                        groups.append(Group.objects.get(name='PROFILING'))
                    if is_production:
                        groups.append(Group.objects.get(name='PRODUCTION'))
                    if is_inventory:
                        groups.append(Group.objects.get(name='INVENTORY'))
                    if is_pos:
                        groups.append(Group.objects.get(name='POS'))
                    if is_accounting:
                        groups.append(Group.objects.get(name='ACCOUNTING'))

                staff_member.groups.set(groups)

                # Success message
                messages.success(request, "Staff member updated successfully!")
            except Exception as e:
                messages.error(request, f'An error occurred: {str(e)}')

            return render(request, "backoffice/staff/edit_staff.html", {'staff_member': staff_member})

        return render(request, "backoffice/staff/edit_staff.html", {'staff_member': staff_member})
    else:
        return HttpResponseForbidden("You are not authorized to view this page.")

    
@login_required
def staffs(request):
    if request.user.is_admin_user or request.user.is_super_user:
        staffs_list = Account.objects.all()
        paginator = Paginator(staffs_list, 10)  # Show 10 staffs per page

        page_number = request.GET.get('page')
        try:
            page_obj = paginator.get_page(page_number)
        except PageNotAnInteger:
            messages.error(request, "Invalid page number.")
            page_obj = paginator.page(1)
        except EmptyPage:
            messages.error(request, "Page number out of range.")
            page_obj = paginator.page(paginator.num_pages)

        if 'export' in request.GET:
            try:
                return export_staffs_to_excel(staffs_list)
            except Exception as e:
                messages.error(request, f"Error exporting staffs: {str(e)}")

        data = {
            "page_obj": page_obj
        }
        return render(request, "backoffice/staff/staffs.html", data)
    else:
        return HttpResponseForbidden("You are not authorized to view this page.")

def export_staffs_to_excel(staffs_list):
    data = []
    for staff in staffs_list:
        data.append({
            'First Name': staff.first_name,
            'Second Name': staff.second_name,
            'Email': staff.email,
            'Phone Number': staff.phone_number,
            'Gender': staff.gender,
            # 'Date of Birth': staff.dob,
            'Marital Status': staff.marital_status,
            'Employment Type': staff.employment_type,
            'Location': staff.location,
            'Country': staff.country,
            'Status': staff.status,
            'Roles': staff.get_roles(),
            'KRA PIN': staff.kra_pin,
            'Education': staff.education,
            'Nationality': staff.nationality,
            'Next of Kin Name': staff.next_of_kin_name,
            'Next of Kin Phone': staff.next_of_kin_phone,
            'Emergency Contact': staff.emergency_contact,
            'Emergency Contact Name': staff.emergency_contact_name,
            'Emergency Contact Phone': staff.emergency_contact_phone,
            'Emergency Contact Relationship': staff.emergency_contact_relationship,
            'Secondary Emergency Contact Name': staff.secondary_emergency_contact_name,
            'Secondary Emergency Contact Phone': staff.secondary_emergency_contact_phone,
            'Secondary Emergency Contact Relationship': staff.secondary_emergency_contact_relationship,
            'Average Rating': staff.average_rating,
            'Profile Image': staff.profile_image.url if staff.profile_image else None,
            'Documents': staff.documents.url if staff.documents else None,
            'Is Staff': staff.is_staff,
            'Is Admin User': staff.is_admin_user,
            'Is Super User': staff.is_super_user,
            'Is Sales': staff.is_sales,
            'Is Purchases': staff.is_purchases,
            'Is Profiling': staff.is_profiling,
            'Is Production': staff.is_production,
            'Is Inventory': staff.is_inventory,
            'Is POS': staff.is_pos,
            'Is Accounting': staff.is_accounting,
            # 'Created At': staff.created_at,
            # 'Updated At': staff.updated_at
        })

    df = pd.DataFrame(data)
    response = HttpResponse(content_type='application/vnd.ms-excel')
    response['Content-Disposition'] = 'attachment; filename=staffs.xls'
    df.to_excel(response, index=False)
    return response

@login_required
def delete_staff(request, staff_id):
    if request.user.is_admin_user or request.user.is_super_user:
        try:
            staff_member = get_object_or_404(Account, id=staff_id)
            staff_member.delete()
            messages.success(request, "Staff member deleted successfully!")
        except Exception as e:
            messages.error(request, f"Error deleting staff member: {str(e)}")
        return redirect('backoffice:staffs')
    else:
        return HttpResponseForbidden("You are not authorized to delete staff members.")